package sun.java2d.pipe;

import java.awt.geom.AffineTransform;
import java.awt.Shape;
import sun.java2d.loops.ProcessPath;
import java.awt.geom.Path2D;
import java.awt.Polygon;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.RoundRectangle2D;
import sun.java2d.SunGraphics2D;

public abstract class BufferedRenderPipe implements PixelDrawPipe, PixelFillPipe, ShapeDrawPipe, ParallelogramPipe
{
    ParallelogramPipe aapgrampipe;
    static final int BYTES_PER_POLY_POINT = 8;
    static final int BYTES_PER_SCANLINE = 12;
    static final int BYTES_PER_SPAN = 16;
    protected RenderQueue rq;
    protected RenderBuffer buf;
    private BufferedRenderPipe.BufferedDrawHandler drawHandler;
    
    public BufferedRenderPipe(RenderQueue rq) {
        this.aapgrampipe = new BufferedRenderPipe.AAParallelogramPipe();
        this.rq = rq;
        this.buf = rq.getBuffer();
        this.drawHandler = new BufferedRenderPipe.BufferedDrawHandler();
    }
    
    public ParallelogramPipe getAAParallelogramPipe() {
        return this.aapgrampipe;
    }
    
    protected abstract void validateContext(SunGraphics2D p0);
    
    protected abstract void validateContextAA(SunGraphics2D p0);
    
    @Override
    public void drawLine(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        int transX = sunGraphics2D.transX;
        int transY = sunGraphics2D.transY;
        this.rq.lock();
        try {
            this.validateContext(sunGraphics2D);
            this.rq.ensureCapacity(20);
            this.buf.putInt(10);
            this.buf.putInt(n + transX);
            this.buf.putInt(n2 + transY);
            this.buf.putInt(n3 + transX);
            this.buf.putInt(n4 + transY);
        }
        finally {
            this.rq.unlock();
        }
    }
    
    @Override
    public void drawRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        this.rq.lock();
        try {
            this.validateContext(sunGraphics2D);
            this.rq.ensureCapacity(20);
            this.buf.putInt(11);
            this.buf.putInt(n + sunGraphics2D.transX);
            this.buf.putInt(n2 + sunGraphics2D.transY);
            this.buf.putInt(n3);
            this.buf.putInt(n4);
        }
        finally {
            this.rq.unlock();
        }
    }
    
    @Override
    public void fillRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        this.rq.lock();
        try {
            this.validateContext(sunGraphics2D);
            this.rq.ensureCapacity(20);
            this.buf.putInt(20);
            this.buf.putInt(n + sunGraphics2D.transX);
            this.buf.putInt(n2 + sunGraphics2D.transY);
            this.buf.putInt(n3);
            this.buf.putInt(n4);
        }
        finally {
            this.rq.unlock();
        }
    }
    
    @Override
    public void drawRoundRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        this.draw(sunGraphics2D, new RoundRectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4, (float)n5, (float)n6));
    }
    
    @Override
    public void fillRoundRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        this.fill(sunGraphics2D, new RoundRectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4, (float)n5, (float)n6));
    }
    
    @Override
    public void drawOval(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        this.draw(sunGraphics2D, new Ellipse2D.Float((float)n, (float)n2, (float)n3, (float)n4));
    }
    
    @Override
    public void fillOval(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        this.fill(sunGraphics2D, new Ellipse2D.Float((float)n, (float)n2, (float)n3, (float)n4));
    }
    
    @Override
    public void drawArc(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        this.draw(sunGraphics2D, new Arc2D.Float((float)n, (float)n2, (float)n3, (float)n4, (float)n5, (float)n6, 0));
    }
    
    @Override
    public void fillArc(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        this.fill(sunGraphics2D, new Arc2D.Float((float)n, (float)n2, (float)n3, (float)n4, (float)n5, (float)n6, 2));
    }
    
    protected void drawPoly(final SunGraphics2D sunGraphics2D, final int[] array, final int[] array2, final int n, final boolean b) {
        if (array == null || array2 == null) {
            throw new NullPointerException("coordinate array");
        }
        if (array.length < n || array2.length < n) {
            throw new ArrayIndexOutOfBoundsException("coordinate array");
        }
        if (n < 2) {
            return;
        }
        if (n == 2 && !b) {
            this.drawLine(sunGraphics2D, array[0], array2[0], array[1], array2[1]);
            return;
        }
        this.rq.lock();
        try {
            this.validateContext(sunGraphics2D);
            int n2 = 20 + n * 8;
            if (n2 <= this.buf.capacity()) {
                if (n2 > this.buf.remaining()) {
                    this.rq.flushNow();
                }
                this.buf.putInt(12);
                this.buf.putInt(n);
                this.buf.putInt(b ? 1 : 0);
                this.buf.putInt(sunGraphics2D.transX);
                this.buf.putInt(sunGraphics2D.transY);
                this.buf.put(array, 0, n);
                this.buf.put(array2, 0, n);
            }
            else {
                this.rq.flushAndInvokeNow(new Runnable() {
                    @Override
                    public void run() {
                        BufferedRenderPipe.this.drawPoly(array, array2, n, b, sunGraphics2D.transX, sunGraphics2D.transY);
                    }
                });
            }
        }
        finally {
            this.rq.unlock();
        }
        return;
    }
    
    protected abstract void drawPoly(int[] p0, int[] p1, int p2, boolean p3, int p4, int p5);
    
    @Override
    public void drawPolyline(SunGraphics2D sunGraphics2D, int[] array, int[] array2, int n) {
        this.drawPoly(sunGraphics2D, array, array2, n, false);
    }
    
    @Override
    public void drawPolygon(SunGraphics2D sunGraphics2D, int[] array, int[] array2, int n) {
        this.drawPoly(sunGraphics2D, array, array2, n, true);
    }
    
    @Override
    public void fillPolygon(SunGraphics2D sunGraphics2D, int[] xpoints, int[] ypoints, int npoints) {
        this.fill(sunGraphics2D, new Polygon(xpoints, ypoints, npoints));
    }
    
    protected void drawPath(SunGraphics2D sunGraphics2D, Path2D.Float p2df, int transX, int transY) {
        this.rq.lock();
        try {
            this.validateContext(sunGraphics2D);
            this.drawHandler.validate(sunGraphics2D);
            ProcessPath.drawPath(this.drawHandler, p2df, transX, transY);
        }
        finally {
            this.rq.unlock();
        }
    }
    
    protected void fillPath(SunGraphics2D sunGraphics2D, Path2D.Float p2df, int transX, int transY) {
        this.rq.lock();
        try {
            this.validateContext(sunGraphics2D);
            this.drawHandler.validate(sunGraphics2D);
            this.drawHandler.startFillPath();
            ProcessPath.fillPath(this.drawHandler, p2df, transX, transY);
            this.drawHandler.endFillPath();
        }
        finally {
            this.rq.unlock();
        }
    }
    
    private native int fillSpans(RenderQueue p0, long p1, int p2, int p3, SpanIterator p4, long p5, int p6, int p7);
    
    protected void fillSpans(SunGraphics2D sunGraphics2D, SpanIterator spanIterator, int n, int n2) {
        this.rq.lock();
        try {
            this.validateContext(sunGraphics2D);
            this.rq.ensureCapacity(24);
            this.buf.position((long)this.fillSpans(this.rq, this.buf.getAddress(), this.buf.position(), this.buf.capacity(), spanIterator, spanIterator.getNativeIterator(), n, n2));
        }
        finally {
            this.rq.unlock();
        }
    }
    
    @Override
    public void fillParallelogram(SunGraphics2D sunGraphics2D, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10) {
        this.rq.lock();
        try {
            this.validateContext(sunGraphics2D);
            this.rq.ensureCapacity(28);
            this.buf.putInt(22);
            this.buf.putFloat((float)n5);
            this.buf.putFloat((float)n6);
            this.buf.putFloat((float)n7);
            this.buf.putFloat((float)n8);
            this.buf.putFloat((float)n9);
            this.buf.putFloat((float)n10);
        }
        finally {
            this.rq.unlock();
        }
    }
    
    @Override
    public void drawParallelogram(SunGraphics2D sunGraphics2D, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10, double n11, double n12) {
        this.rq.lock();
        try {
            this.validateContext(sunGraphics2D);
            this.rq.ensureCapacity(36);
            this.buf.putInt(15);
            this.buf.putFloat((float)n5);
            this.buf.putFloat((float)n6);
            this.buf.putFloat((float)n7);
            this.buf.putFloat((float)n8);
            this.buf.putFloat((float)n9);
            this.buf.putFloat((float)n10);
            this.buf.putFloat((float)n11);
            this.buf.putFloat((float)n12);
        }
        finally {
            this.rq.unlock();
        }
    }
    
    @Override
    public void draw(SunGraphics2D sunGraphics2D, Shape shape) {
        if (sunGraphics2D.strokeState == 0) {
            if (shape instanceof Polygon && sunGraphics2D.transformState < 3) {
                Polygon polygon = (Polygon)shape;
                this.drawPolygon(sunGraphics2D, polygon.xpoints, polygon.ypoints, polygon.npoints);
                return;
            }
            Path2D.Float float1;
            int transX;
            int transY;
            if (sunGraphics2D.transformState <= 1) {
                if ((shape instanceof Path2D.Float)) {
                    float1 = (Path2D.Float)shape;
                }
                else {
                    float1 = new Path2D.Float(shape);
                }
                transX = sunGraphics2D.transX;
                transY = sunGraphics2D.transY;
            }
            else {
                float1 = new Path2D.Float(shape, sunGraphics2D.transform);
                transX = 0;
                transY = 0;
            }
            this.drawPath(sunGraphics2D, float1, transX, transY);
        }
        else if (sunGraphics2D.strokeState < 3) {
            ShapeSpanIterator strokeSpans = LoopPipe.getStrokeSpans(sunGraphics2D, shape);
            try {
                this.fillSpans(sunGraphics2D, strokeSpans, 0, 0);
            }
            finally {
                strokeSpans.dispose();
            }
        }
        else {
            this.fill(sunGraphics2D, sunGraphics2D.stroke.createStrokedShape(shape));
        }
    }
    
    @Override
    public void fill(SunGraphics2D sunGraphics2D, Shape shape) {
        if (sunGraphics2D.strokeState == 0) {
            Path2D.Float float1;
            int transX2;
            int transY2;
            if (sunGraphics2D.transformState <= 1) {
                if ((shape instanceof Path2D.Float)) {
                    float1 = (Path2D.Float)shape;
                }
                else {
                    float1 = new Path2D.Float(shape);
                }
                transX2 = sunGraphics2D.transX;
                transY2 = sunGraphics2D.transY;
            }
            else {
                float1 = new Path2D.Float(shape, sunGraphics2D.transform);
                transX2 = 0;
                transY2 = 0;
            }
            this.fillPath(sunGraphics2D, float1, transX2, transY2);
            return;
        }
        else {
            AffineTransform transform;
            int transX;
            int transY;
            if (sunGraphics2D.transformState <= 1) {
                transform = null;
                transX = sunGraphics2D.transX;
                transY = sunGraphics2D.transY;
            }
            else {
                transform = sunGraphics2D.transform;
                transY = (transX = 0);
            }
            ShapeSpanIterator fillSSI = LoopPipe.getFillSSI(sunGraphics2D);
            try {
                Region compClip = sunGraphics2D.getCompClip();
                fillSSI.setOutputAreaXYXY(compClip.getLoX() - transX, compClip.getLoY() - transY, compClip.getHiX() - transX, compClip.getHiY() - transY);
                fillSSI.appendPath(shape.getPathIterator(transform));
                this.fillSpans(sunGraphics2D, fillSSI, transX, transY);
            }
            finally {
                fillSSI.dispose();
            }
            return;
        }
    }
    
    private class BufferedDrawHandler extends ProcessPath.DrawHandler
    {
        private int scanlineCount;
        private int scanlineCountIndex;
        private int remainingScanlines;
        
        BufferedDrawHandler() {
            super(0, 0, 0, 0);
        }
        
        void validate(SunGraphics2D sunGraphics2D) {
            Region compClip = sunGraphics2D.getCompClip();
            this.setBounds(compClip.getLoX(), compClip.getLoY(), compClip.getHiX(), compClip.getHiY(), sunGraphics2D.strokeHint);
        }
        
        @Override
        public void drawLine(int n, int n2, int n3, int n4) {
            BufferedRenderPipe.this.rq.ensureCapacity(20);
            BufferedRenderPipe.this.buf.putInt(10);
            BufferedRenderPipe.this.buf.putInt(n);
            BufferedRenderPipe.this.buf.putInt(n2);
            BufferedRenderPipe.this.buf.putInt(n3);
            BufferedRenderPipe.this.buf.putInt(n4);
        }
        
        @Override
        public void drawPixel(int n, int n2) {
            BufferedRenderPipe.this.rq.ensureCapacity(12);
            BufferedRenderPipe.this.buf.putInt(13);
            BufferedRenderPipe.this.buf.putInt(n);
            BufferedRenderPipe.this.buf.putInt(n2);
        }
        
        private void resetFillPath() {
            BufferedRenderPipe.this.buf.putInt(14);
            this.scanlineCountIndex = BufferedRenderPipe.this.buf.position();
            BufferedRenderPipe.this.buf.putInt(0);
            this.scanlineCount = 0;
            this.remainingScanlines = BufferedRenderPipe.this.buf.remaining() / 12;
        }
        
        private void updateScanlineCount() {
            BufferedRenderPipe.this.buf.putInt(this.scanlineCountIndex, this.scanlineCount);
        }
        
        public void startFillPath() {
            BufferedRenderPipe.this.rq.ensureCapacity(20);
            this.resetFillPath();
        }
        
        @Override
        public void drawScanline(int n, int n2, int n3) {
            if (this.remainingScanlines == 0) {
                this.updateScanlineCount();
                BufferedRenderPipe.this.rq.flushNow();
                this.resetFillPath();
            }
            BufferedRenderPipe.this.buf.putInt(n);
            BufferedRenderPipe.this.buf.putInt(n2);
            BufferedRenderPipe.this.buf.putInt(n3);
            ++this.scanlineCount;
            --this.remainingScanlines;
        }
        
        public void endFillPath() {
            this.updateScanlineCount();
        }
    }
    
    private class AAParallelogramPipe implements ParallelogramPipe
    {
        @Override
        public void fillParallelogram(SunGraphics2D sunGraphics2D, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10) {
            BufferedRenderPipe.this.rq.lock();
            try {
                BufferedRenderPipe.this.validateContextAA(sunGraphics2D);
                BufferedRenderPipe.this.rq.ensureCapacity(28);
                BufferedRenderPipe.this.buf.putInt(23);
                BufferedRenderPipe.this.buf.putFloat((float)n5);
                BufferedRenderPipe.this.buf.putFloat((float)n6);
                BufferedRenderPipe.this.buf.putFloat((float)n7);
                BufferedRenderPipe.this.buf.putFloat((float)n8);
                BufferedRenderPipe.this.buf.putFloat((float)n9);
                BufferedRenderPipe.this.buf.putFloat((float)n10);
            }
            finally {
                BufferedRenderPipe.this.rq.unlock();
            }
        }
        
        @Override
        public void drawParallelogram(SunGraphics2D sunGraphics2D, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10, double n11, double n12) {
            BufferedRenderPipe.this.rq.lock();
            try {
                BufferedRenderPipe.this.validateContextAA(sunGraphics2D);
                BufferedRenderPipe.this.rq.ensureCapacity(36);
                BufferedRenderPipe.this.buf.putInt(16);
                BufferedRenderPipe.this.buf.putFloat((float)n5);
                BufferedRenderPipe.this.buf.putFloat((float)n6);
                BufferedRenderPipe.this.buf.putFloat((float)n7);
                BufferedRenderPipe.this.buf.putFloat((float)n8);
                BufferedRenderPipe.this.buf.putFloat((float)n9);
                BufferedRenderPipe.this.buf.putFloat((float)n10);
                BufferedRenderPipe.this.buf.putFloat((float)n11);
                BufferedRenderPipe.this.buf.putFloat((float)n12);
            }
            finally {
                BufferedRenderPipe.this.rq.unlock();
            }
        }
    }
}
